package cn.com.qmth.stmms.admin.exam;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import cn.com.qmth.stmms.biz.campus.model.Campus;
import cn.com.qmth.stmms.biz.campus.service.CampusService;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.MarkGroup;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.exam.service.MarkGroupService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.model.MarkLibrary;
import cn.com.qmth.stmms.biz.mark.model.MarkSpecialTag;
import cn.com.qmth.stmms.biz.mark.model.MarkTrack;
import cn.com.qmth.stmms.biz.mark.query.MarkLibrarySearchQuery;
import cn.com.qmth.stmms.biz.mark.service.MarkLibraryService;
import cn.com.qmth.stmms.biz.mark.service.MarkSpecialTagService;
import cn.com.qmth.stmms.biz.mark.service.MarkTrackService;
import cn.com.qmth.stmms.biz.utils.ScoreItem;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import cn.com.qmth.stmms.common.utils.PictureUrlBuilder;

@Controller("inspectedController")
@RequestMapping("/admin/exam/inspected")
public class InspectedController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(InspectedController.class);

    @Autowired
    private MarkLibraryService libraryService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private CampusService campusService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private MarkTrackService markTrackService;

    @Autowired
    private MarkSpecialTagService markSpecialTagService;

    @Autowired
    private MarkGroupService groupService;

    @Value("${slice.image.server}")
    private String sliceServer;

    @RequestMapping("/start")
    @ResponseBody
    public ModelAndView start(HttpServletRequest request, @RequestParam String subjectCode,
            @RequestParam Integer groupNumber) {
        int examId = getSessionExamId(request);
        if (examId > 0) {
            ModelAndView view = new ModelAndView("modules/exam/inspected");
            List<Integer> ids = new ArrayList<Integer>();
            MarkLibrarySearchQuery query = new MarkLibrarySearchQuery();
            query.setExamId(examId);
            query.setSubjectCode(subjectCode);
            query.setGroupNumber(groupNumber);
            query.setPageNumber(1);
            query.setPageSize(1000);
            query.addStatus(LibraryStatus.MARKED);
            query = libraryService.findByQuery(query);
            while (query.getCurrentCount() > 0) {
                for (MarkLibrary library : query.getResult()) {
                    ids.add(library.getId());
                }
                query.setPageNumber(query.getPageNumber() + 1);
                query = libraryService.findByQuery(query);
            }
            view.addObject("sliceServer", sliceServer);
            view.addObject("ids", StringUtils.join(ids, ","));
            return view;
        } else {
            return new ModelAndView("redirect:/admin/exam/list");
        }
    }

    @RequestMapping("/info")
    @ResponseBody
    public Object info(@RequestParam Integer libraryId) {
        MarkLibrary library = libraryService.findById(libraryId);
        ExamStudent student = studentService.findById(library.getStudentId());
        JSONObject result = new JSONObject();
        result.accumulate("id", libraryId);
        result.accumulate("studentId", library.getStudentId());
        result.accumulate("subjectCode", library.getSubjectCode());
        result.accumulate("subjectName", student.getSubjectName());
        Marker marker = markerService.findById(library.getMarkerId());
        result.accumulate("markerName", marker.getLoginName());
        result.accumulate("markerScore", library.getMarkerScore());

        JSONArray array = new JSONArray();
        List<ExamQuestion> questions = questionService.findByExamAndSubjectAndObjectiveAndGroupNumber(
                library.getExamId(), library.getSubjectCode(), false, library.getGroupNumber());
        List<ScoreItem> scores = library.getScoreList();
        for (int i = 0; i < questions.size(); i++) {
            ExamQuestion question = questions.get(i);
            JSONObject obj = new JSONObject();
            obj.accumulate("questionNumber", question.getMainTitle() + " " + question.getQuestionNumber());
            obj.accumulate("score", scores.get(i).getScore());
            array.add(obj);
        }
        result.accumulate("questions", array);

        MarkGroup group = groupService.findOne(student.getExamId(), student.getSubjectCode(), library.getGroupNumber());
        Campus campus = campusService.findBySchoolAndName(student.getSchoolId(), student.getCampusName());
        List<String> picUrls = PictureUrlBuilder.getSliceUrls(student.getExamId(), campus.getId(),
                student.getSubjectCode(), student.getExamNumber(), student.getSliceCount());
        List<MarkTrack> markTracks = markTrackService.findByLibraryId(library.getId());
        List<MarkSpecialTag> markSpecialTagList = markSpecialTagService.findByLibraryId(library.getId());
        result.accumulate("picUrls", picUrls);
        result.accumulate("pictureConfig", group.getPictureConfigList());
        result.accumulate("markTracks", markTracks);
        result.accumulate("markSpecialTagList", markSpecialTagList);
        return result;
    }

    @RequestMapping("/save")
    @ResponseBody
    public Object save(HttpServletRequest request, @RequestParam Integer libraryId) {
        MarkLibrary library = libraryService.findById(libraryId);
        if (library != null && library.getStatus().equals(LibraryStatus.MARKED)) {
            library.setStatus(LibraryStatus.INSPECTED);
            libraryService.save(library);
            return true;
        } else {
            return false;
        }
    }
}
